@@ -26,13 +26,22 @@ read.rm5 <- function(file, sep=",", quote = "\"",
     title <- tmp[[1]][length(tmp[[1]])]
   }
   ##
+  ## Check number of fields in input data file
+  ##
+  if (length(unique(count.fields(file, sep=sep, quote=quote, comment.char=""))) != 1)
+    stop("Input file has different number of elements. Probably your export data file contains field 'Risk of bias tables'. Please create a new export data file in RevMan 5 without this field.")
+  ##
   tdata <- read.table(file, header=TRUE,
                       sep=sep, quote=quote,
                       comment.char="")
   ##
   nam <- names(tdata)
+  
   ##
-  ##print(nam)
+  ## Overcome problems to import files with UTF-8, byte order mark encoding
+  ## (see http://en.wikipedia.org/wiki/Byte_order_mark)
+  ##
+  nam[grep("Comparison.Number$", nam)] <- "Comparison.Number"
   ##
   if (!all(c("Comparison.Number", "Outcome.Number", "Subgroup.Number") %in% nam))
     stop("Mandatory fields 'Comparison Number', 'Outcome Number',",
@@ -50,6 +59,15 @@ read.rm5 <- function(file, sep=",", quote = "\"",
   nam[nam=="Effect.Measure"] <- "sm"
   nam[nam=="Analysis.Model"] <- "model"
   ##
+  ## Variable 'Totals' not included in data frame 
+  ## Variable 'Study.Confidence.Interval' not included in data frame 
+  ## Variable 'Total.Confidence.Interval' not included in data frame 
+  ## Variable 'Test.for.subgroup.differences' not included in data frame 
+  ##
+  nam[nam=="Swap.event.and.non.event"] <- "swap.events"
+  nam[nam=="Entered.data.are.on.log.scale..Generic.Inverse.Variance.only."] <- "logscale"
+  nam[nam=="Enter.number.of.participants..Generic.Inverse.Variance..for.display.only."] <- "enter.n"
+  ##
   nam[nam=="Events.1"] <- "event.e"
   nam[nam=="Mean.1"] <- "mean.e"
   nam[nam=="SD.1"] <- "sd.e"
@@ -62,6 +80,7 @@ read.rm5 <- function(file, sep=",", quote = "\"",
   ##
   nam[nam=="O.E"] <- "O.E"
   nam[nam=="Var"] <- "V"
+  ##
   nam[nam=="Effect.Estimate"] <- "TE"
   nam[nam=="SE"] <- "seTE"
   ##
@@ -70,13 +89,17 @@ read.rm5 <- function(file, sep=",", quote = "\"",
   ##
   nam[nam=="Weight"]   <- "weight"
   ##
+  ## Variable 'Q' is Q
   nam[nam=="P.Q."]  <- "pval.Q"
   nam[nam=="I..Q."] <- "I2"
   nam[nam=="Tau."]  <- "tau2"
   ##
+  ## Variable 'Z' is Z
   nam[nam=="P.Z."]     <- "pval.TE"
+  ## Variable 'Qint' is Qint
   nam[nam=="P.Qint."]  <- "pval.Qint"
   nam[nam=="I..Qint."] <- "I2.Qint"
+  ## Variable 'df' is df
   ##
   nam[nam=="Group.Label.1"]     <- "label.e"
   nam[nam=="Group.Label.2"]     <- "label.c"
@@ -114,6 +137,15 @@ read.rm5 <- function(file, sep=",", quote = "\"",
   tdata$comb.fixed <- tdata$model=="Fixed"
   tdata$comb.random <- tdata$model=="Random"
   ##
+  tdata$sm[tdata$sm=="Odds Ratio"] <- "OR"
+  tdata$sm[tdata$sm=="Odds Ratio (Non-event)"] <- "OR"
+  tdata$sm[tdata$sm=="Risk Ratio"] <- "RR"
+  tdata$sm[tdata$sm=="Risk Difference"] <- "RD"
+  tdata$sm[tdata$sm=="Mean Difference"] <- "MD"
+  tdata$sm[tdata$sm=="Standardized Mean Difference"] <- "SMD"
+  tdata$sm[tdata$sm=="Std. Mean Difference"] <- "SMD"
+  tdata$sm[tdata$sm=="Hazard Ratio"] <- "HR"
+  ##
   sel.oe <- tdata$method=="EXP_O_E_VAR"
   tdata$method[sel.oe] <- "Peto"
   tdata$sm[sel.oe] <- "OR"
@@ -122,9 +154,31 @@ read.rm5 <- function(file, sep=",", quote = "\"",
   tdata$method[sel.peto] <- "Peto"
   tdata$sm[sel.peto] <- "OR"
   ##
-  tdata$RR.cochrane <- rep(NA, length(tdata$sm))
-  ##tdata$RR.cochrane[tdata$method=="MH" & tdata$sm=="RR"] <- TRUE
-  tdata$RR.cochrane[tdata$sm=="RR"] <- TRUE
+  tdata$event.e <- as.numeric(gsub(",", "", tdata$event.e))
+  tdata$n.e <- as.numeric(gsub(",", "", tdata$n.e))
+  tdata$event.c <- as.numeric(gsub(",", "", tdata$event.c))
+  tdata$n.c <- as.numeric(gsub(",", "", tdata$n.c))
+  tdata$mean.e <- as.numeric(gsub(",", "", tdata$mean.e))
+  tdata$sd.e <- as.numeric(gsub(",", "", tdata$sd.e))
+  tdata$mean.c <- as.numeric(gsub(",", "", tdata$mean.c))
+  tdata$sd.c <- as.numeric(gsub(",", "", tdata$sd.c))
+  if (!is.null(tdata$O.E))
+    tdata$O.E <- as.numeric(gsub(",", "", tdata$O.E))
+  if (!is.null(tdata$V))
+    tdata$V <- as.numeric(gsub(",", "", tdata$V))
+  tdata$TE <- as.numeric(gsub(",", "", tdata$TE))
+  ## No warning concerning infinite values for
+  ## seTE, lower and upper CI bound
+  oldopts <- options(warn=-1)
+  tdata$seTE <- as.numeric(gsub(",", "", tdata$seTE))
+  tdata$lower <- as.numeric(gsub(",", "", tdata$lower))
+  tdata$upper <- as.numeric(gsub(",", "", tdata$upper))
+  options(oldopts)
+  ##
+  tdata$weight <- as.numeric(gsub(",", "", tdata$weight))
+  tdata$Q <- as.numeric(gsub(",", "", tdata$Q))
+  tdata$tau2 <- as.numeric(gsub(",", "", tdata$tau2))
+  tdata$Qint <- as.numeric(gsub(",", "", tdata$Qint))
   ##
   diffcomp <- c(1, diff(tdata$comp.no))
   diffoutc <- c(1, diff(tdata$outcome.no))
@@ -132,7 +186,7 @@ read.rm5 <- function(file, sep=",", quote = "\"",
   ##
   sel.comp  <- diffcomp != 0
   sel.outc  <- diffoutc != 0 & !sel.comp
-  sel.grp   <- diffgrp  != 0 & !sel.comp & !sel.outc
+  sel.grp   <- tdata$type != "" & diffgrp  != 0 & !sel.comp & !sel.outc
   sel.study <- !sel.comp & !sel.outc & !sel.grp
   ##
   ##print(data.frame(comp.no=tdata$comp.no,
@@ -145,6 +199,13 @@ read.rm5 <- function(file, sep=",", quote = "\"",
   ##
   ##print(names(tdata))
   ##
+  ##print(cbind(tdata$comp.no, sel.comp,
+  ##            tdata$outcome.no, sel.outc,
+  ##            tdata$group.no, sel.grp,
+  ##            sel.study,
+  ##            substring(tdata$author, 1, 20))
+  ##      )
+  ##
   res <- list(title=title,
               comparison=
               data.frame(comp.no= selvar(tdata$comp.no, sel.comp),
@@ -187,11 +248,14 @@ read.rm5 <- function(file, sep=",", quote = "\"",
                          pval.Q.w = selvar(tdata$pval.Qint, sel.outc),
                          I2.w     = selvar(tdata$I2.Qint, sel.outc),
                          ##
+                         swap.events = selvar(tdata$swap.events, sel.outc),
+                         enter.n     = selvar(tdata$enter.n, sel.outc),
+                         logscale    = selvar(tdata$logscale, sel.outc),
+                         ##
                          label.e     = selvar(tdata$label.e, sel.outc),
                          label.c     = selvar(tdata$label.c, sel.outc),
                          label.left  = selvar(tdata$label.left, sel.outc),
-                         label.right = selvar(tdata$label.right, sel.outc),
-                         RR.cochrane = selvar(tdata$RR.cochrane, sel.outc)
+                         label.right = selvar(tdata$label.right, sel.outc)
                          ),
               group=
               if (sum(sel.grp) > 0 ){
@@ -265,7 +329,7 @@ read.rm5 <- function(file, sep=",", quote = "\"",
   res2$studlab <- nachar(as.character(res2$studlab))
   if (is.data.frame(res$group)){
     res2$grplab <- nachar(as.character(res2$grplab))
-    ## Recode special signs:
+    ## Replace some XML code:
     res2$grplab <- gsub("\\&lt;", "<", res2$grplab)
     res2$grplab <- gsub("\\&gt;", ">", res2$grplab)
   }
@@ -309,35 +373,21 @@ read.rm5 <- function(file, sep=",", quote = "\"",
     }
   }
   
+  
+  ##res2$TE[res2$sm=="HR"] <- log(res2$TE[res2$sm=="HR"])
+  ##res2$sm[res2$sm=="log HR"] <- "HR"
+
+  iswap.events <- charmatch(tolower(as.character(res2$swap.events)), c("yes", "no"), nomatch=NA)
+  res2$swap.events <- ifelse(iswap.events==1, TRUE, ifelse(iswap.events==2, FALSE, NA))
+  ##
+  ienter.n <- charmatch(tolower(as.character(res2$enter.n)), c("yes", "no"), nomatch=NA)
+  res2$enter.n <- ifelse(ienter.n==1, TRUE, ifelse(ienter.n==2, FALSE, NA))
+  ##
+  ilogscale <- charmatch(tolower(as.character(res2$logscale)), c("no", "yes"), nomatch=NA)
+  res2$logscale <- ifelse(ilogscale==1, TRUE, ifelse(ilogscale==2, FALSE, NA))
+  
   attr(res2, "version") <- packageDescription("meta")$Version
   
-  class(res2) <- c("data.frame", "rm5")
+  class(res2) <- c("rm5", "data.frame")
   res2
 }
-
-
-##  res2$event.e[res2$type == "I" & res2$event.e==0 & res2$n.e==1] <- NA
-##  res2$n.e[res2$type == "I" & res2$n.e==1] <- NA
-##  res2$event.c[res2$type == "I" & res2$event.c==0 & res2$n.c==1] <- NA
-##  res2$n.c[res2$type == "I" & res2$n.c==1] <- NA
-##  res2$mean.c[res2$type == "I" & res2$mean.c==0 & res2$sd.c==0] <- NA
-##  res2$sd.c[res2$type == "I" & res2$sd.c==0] <- NA
-##  res2$O.E[res2$type == "I" & res2$O.E==0 & res2$V==0] <- NA
-##  res2$V[res2$type == "I" & res2$V==0] <- NA
-##  ##
-##  res2$mean.e[res2$type == "D" & res2$mean.e==0 & res2$sd.e==0] <- NA
-##  res2$sd.e[res2$type == "D" & res2$sd.e==0] <- NA
-##  res2$mean.c[res2$type == "D" & res2$mean.c==0 & res2$sd.c==0] <- NA
-##  res2$sd.c[res2$type == "D" & res2$sd.c==0] <- NA
-##  res2$O.E[res2$type == "D" & res2$O.E==0 & res2$V==0] <- NA
-##  res2$V[res2$type == "D" & res2$V==0] <- NA
-##  ##
-##  res2$event.e[res2$type == "C" & res2$event.e==0] <- NA
-##  res2$event.c[res2$type == "C" & res2$event.c==0] <- NA
-##  res2$O.E[res2$type == "C" & res2$O.E==0 & res2$V==0] <- NA
-##  res2$V[res2$type == "C" & res2$V==0] <- NA
-##  ##
-##  res2$mean.e[res2$type == "P" & res2$mean.e==0 & res2$sd.e==0] <- NA
-##  res2$sd.e[res2$type == "P" & res2$sd.e==0] <- NA
-##  res2$mean.c[res2$type == "P" & res2$mean.c==0 & res2$sd.c==0] <- NA
-##  res2$sd.c[res2$type == "P" & res2$sd.c==0] <- NA
